@@ -20,9 +20,11 @@
 #include <utils/plaintextedit/plaintexteditaccessibility.h>
 #include <utils/processreaper.h>
 #include <utils/qtcsettings.h>
+#include <utils/qtcsettings_p.h>
 #include <utils/stylehelper.h>
 #include <utils/temporarydirectory.h>
 #include <utils/terminalcommand.h>
+#include <utils/textcodec.h>
 
 #include <QAccessible>
 #include <QDebug>
@@ -38,10 +40,10 @@
 #include <QStandardPaths>
 #include <QStyle>
 #include <QSurfaceFormat>
-#include <QTextCodec>
 #include <QTextStream>
 #include <QThreadPool>
 #include <QTranslator>
+#include <QtVersion>
 
 #include <iterator>
 #include <optional>
@@ -64,6 +66,7 @@ Q_LOGGING_CATEGORY(sentryLog, "qtc.sentry", QtWarningMsg)
 
 using namespace ExtensionSystem;
 using namespace Utils;
+using namespace Utils::Internal;
 
 enum { OptionIndent = 4, DescriptionIndent = 34 };
 
@@ -533,6 +536,11 @@ void configureSentry(const AppInfo &appInfo, bool crashReportingEnabled)
     sentry_options_set_release(options, release.toUtf8().constData());
     sentry_options_set_debug(options, sentryLog().isDebugEnabled() ? 1 : 0);
     sentry_init(options);
+    sentry_set_tag("qt.version", qVersion());
+    sentry_set_tag("qt.architecture", QSysInfo::buildCpuArchitecture().toUtf8().constData());
+    sentry_set_tag("qtcreator.compiler", Utils::compilerString().toUtf8().constData());
+    if (!Utils::appInfo().revision.isEmpty())
+        sentry_set_tag("qtcreator.revision", Utils::appInfo().revision.toUtf8().constData());
 }
 #endif
 
@@ -773,7 +781,7 @@ int main(int argc, char **argv)
     const QStringList installPluginPaths = getInstallPluginPaths();
     // Re-setup install settings for real
     setupInstallSettings(options.installSettingsPath);
-    QtcSettings *settings = createUserSettings();
+    QtcSettings *userSettings = createUserSettings();
     QtcSettings *installSettings
         = new QtcSettings(QSettings::IniFormat,
                           QSettings::SystemScope,
@@ -788,7 +796,9 @@ int main(int argc, char **argv)
                               INSTALL_SETTINGS_OPTION,
                               Core::Constants::IDE_SETTINGSVARIANT_STR));
     }
-    TerminalCommand::setSettings(settings);
+
+    SettingsSetup::setupSettings(userSettings, installSettings);
+
     setPixmapCacheLimit();
     loadFonts();
 
@@ -821,7 +831,7 @@ int main(int argc, char **argv)
     info.plugins = (appDirPath / RELATIVE_PLUGIN_PATH).cleanPath();
     info.userPluginsRoot = userPluginsRoot();
     info.resources = (appDirPath / RELATIVE_DATA_PATH).cleanPath();
-    info.userResources = userResourcePath(settings->fileName(), Constants::IDE_ID);
+    info.userResources = userResourcePath(userSettings->fileName(), Constants::IDE_ID);
     info.libexec = (appDirPath / RELATIVE_LIBEXEC_PATH).cleanPath();
     // sync with src\tools\qmlpuppet\qmlpuppet\qmlpuppet.cpp -> QString crashReportsPath()
     info.crashReports = info.userResources / "crashpad_reports";
@@ -834,7 +844,7 @@ int main(int argc, char **argv)
         Core::Constants::IDE_DISPLAY_NAME, CrashHandlerSetup::EnableRestart, info.libexec.path());
 
     // depends on AppInfo and QApplication being created
-    const bool crashReportingEnabled = settings->value("CrashReportingEnabled", false).toBool();
+    const bool crashReportingEnabled = userSettings->value("CrashReportingEnabled", false).toBool();
 
 #if defined(ENABLE_CRASHPAD)
     startCrashpad(info, crashReportingEnabled);
@@ -846,18 +856,14 @@ int main(int argc, char **argv)
 
     PluginManager pluginManager;
     PluginManager::setPluginIID(QLatin1String("org.qt-project.Qt.QtCreatorPlugin"));
-    PluginManager::setInstallSettings(installSettings);
-    PluginManager::setSettings(settings);
     PluginManager::startProfiling();
 
-    BaseAspect::setQtcSettings(settings);
-
     QTranslator translator;
     QTranslator qtTranslator;
     QStringList uiLanguages = QLocale::system().uiLanguages();
     const QString overrideLanguage = options.hasTestOption
                                          ? QString("C") // force built-in when running tests
-                                         : settings->value("General/OverrideLanguage").toString();
+                                         : userSettings->value("General/OverrideLanguage").toString();
     if (!overrideLanguage.isEmpty())
         uiLanguages.prepend(overrideLanguage);
     if (!options.uiLanguage.isEmpty())
@@ -887,9 +893,9 @@ int main(int argc, char **argv)
         }
     }
 
-    QByteArray overrideCodecForLocale = settings->value("General/OverrideCodecForLocale").toByteArray();
+    QByteArray overrideCodecForLocale = userSettings->value("General/OverrideCodecForLocale").toByteArray();
     if (!overrideCodecForLocale.isEmpty())
-        QTextCodec::setCodecForLocale(QTextCodec::codecForName(overrideCodecForLocale));
+        TextEncoding::setEncodingForLocale(overrideCodecForLocale);
 
     app.setDesktopFileName(IDE_APP_ID);
 
@@ -1007,11 +1013,11 @@ int main(int argc, char **argv)
     QObject::connect(&app, &SharedTools::QtSingleApplication::messageReceived,
                      &pluginManager, &PluginManager::remoteArguments);
 
-    QObject::connect(&app, SIGNAL(fileOpenRequest(QString)), coreplugin->plugin(),
-                     SLOT(fileOpenRequest(QString)));
-
     // shutdown plugin manager on the exit
-    QObject::connect(&app, &QCoreApplication::aboutToQuit, &pluginManager, &PluginManager::shutdown);
+    QObject::connect(&app, &QCoreApplication::aboutToQuit, &pluginManager, [] {
+        PluginManager::shutdown();
+        SettingsSetup::destroySettings();
+    });
 
     if (Utils::HostOsInfo::isWindowsHost()) {
         // Workaround for QTBUG-130696 and QTCREATORBUG-31890
