@@ -437,9 +437,11 @@ static Utils::Result<QFuture<R>> createJob(
         if (handleErrors == Errors::Handle && type == "error") {
             const QString err = map.value("Error", QString{}).toString();
             const QString errType = map.value("ErrorType", QString{}).toString();
-            if (errType == "ENOENT") {
+            const int errNo = map.value("Errno", -1).toInt();
+
+            if (errType == "Errno") {
                 promise->setException(
-                    std::make_exception_ptr(std::system_error(ENOENT, std::generic_category())));
+                    std::make_exception_ptr(std::system_error(errNo, std::generic_category())));
                 promise->finish();
             } else if (errType == "NormalExit") {
                 promise->setException(std::make_exception_ptr(std::runtime_error("NormalExit")));
@@ -693,6 +695,17 @@ Utils::Result<QFuture<void>> Client::copyFile(const QString &source, const QStri
         "copyfileresult");
 }
 
+Utils::Result<QFuture<void>> Client::createSymLink(const QString &source, const QString &symLink)
+{
+    return createVoidJob(
+        d.get(),
+        QCborMap{
+                 {"Type", "createsymlink"},
+                 {"CreateSymLink", QCborMap{{"Source", source}, {"SymLink", symLink}}},
+                 },
+        "createsymlinkresult");
+}
+
 Utils::Result<QFuture<void>> Client::renameFile(const QString &source, const QString &target)
 {
     return createVoidJob(
@@ -718,6 +731,20 @@ Utils::Result<QFuture<FilePath>> Client::createTempFile(const QString &path)
         });
 }
 
+Utils::Result<QFuture<FilePath>> Client::createTempDir(const QString &path)
+{
+    return createJob<FilePath>(
+        d.get(),
+        QCborMap{{"Type", "createtempdir"}, {"Path", path}},
+        [](QVariantMap map, QPromise<FilePath> &promise) {
+            ASSERT_TYPE("createtempdirresult");
+
+            promise.addResult(FilePath::fromUserInput(map.value("Path").toString()));
+
+            return JobResult::Done;
+        });
+}
+
 /*
 Convert QFileDevice::Permissions to Unix chmod flags.
 The mode is copied from system libraries.
