@@ -41,7 +41,6 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <projectexplorer/projectexplorertr.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/sysrootkitaspect.h>
 #include <projectexplorer/target.h>
@@ -141,7 +140,7 @@ private:
         const QStringList &configurationArguments = QStringList());
     void updatePackageManagerAutoSetup(CMakeConfig &initialList);
 
-    CMakeBuildConfiguration *m_buildConfig;
+    QPointer<CMakeBuildConfiguration> m_buildConfig;
     QTreeView *m_configView;
     ConfigModel *m_configModel;
     CategorySortFilterModel *m_configFilterModel;
@@ -400,7 +399,12 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
             m_buildConfig->initialCMakeArguments.cmakeConfiguration());
     }
 
-    connect(bs, &BuildSystem::parsingFinished, this, [this, bs] {
+    connect(bs, &BuildSystem::parsingFinished, this, [this] {
+        QTC_ASSERT(m_buildConfig, return);
+        CMakeBuildSystem *bs = m_buildConfig->cmakeBuildSystem();
+        QTC_ASSERT(bs, return);
+        if (bs->isDestructing())
+            return;
         const CMakeConfig config = bs->configurationFromCMake();
         const TriState qmlDebugSetting = m_buildConfig->qmlDebugging();
         bool qmlDebugConfig = CMakeBuildConfiguration::hasQmlDebugging(config);
@@ -527,13 +531,11 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
             &Utils::BaseAspect::labelLinkActivated,
             this,
             [this](const QString &) {
-                const CMakeTool *tool = CMakeKitAspect::cmakeTool(m_buildConfig->kit());
-                CMakeTool::openCMakeHelpUrl(tool, "%1/manual/cmake.1.html#options");
+                CMakeKitAspect::openCMakeHelpUrl(m_buildConfig->kit(), "%1/manual/cmake.1.html#options");
             });
     connect(&bc->additionalCMakeOptions,
             &Utils::BaseAspect::labelLinkActivated, this, [this](const QString &) {
-                const CMakeTool *tool = CMakeKitAspect::cmakeTool(m_buildConfig->kit());
-                CMakeTool::openCMakeHelpUrl(tool, "%1/manual/cmake.1.html#options");
+                CMakeKitAspect::openCMakeHelpUrl(m_buildConfig->kit(), "%1/manual/cmake.1.html#options");
             });
 
     if (HostOsInfo::isMacHost())
@@ -565,14 +567,13 @@ void CMakeBuildSettingsWidget::batchEditConfiguration()
        "&lt;type&gt; can have one of the following values: FILEPATH, PATH, BOOL, INTERNAL, or STRING.<br/>"
                       "To unset a variable, use -U&lt;variable&gt;.<br/>"));
     connect(label, &QLabel::linkActivated, this, [this](const QString &) {
-        const CMakeTool *tool = CMakeKitAspect::cmakeTool(m_buildConfig->kit());
-        CMakeTool::openCMakeHelpUrl(tool, "%1/manual/cmake-variables.7.html");
+        CMakeKitAspect::openCMakeHelpUrl(m_buildConfig->kit(), "%1/manual/cmake-variables.7.html");
     });
     editor->setMinimumSize(800, 200);
 
     auto chooser = new Utils::VariableChooser(dialog);
     chooser->addSupportedWidget(editor);
-    chooser->addMacroExpanderProvider([this] { return m_buildConfig->macroExpander(); });
+    chooser->addMacroExpanderProvider({this, [this] { return m_buildConfig->macroExpander(); }});
 
     auto buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
 
@@ -724,9 +725,15 @@ void CMakeBuildSettingsWidget::updateInitialCMakeArguments(bool fromReconfigure)
 
 void CMakeBuildSettingsWidget::kitCMakeConfiguration()
 {
+    using namespace Layouting;
     m_buildConfig->kit()->blockNotification();
 
     auto dialog = new QDialog(this);
+    auto deleteLater = [dialog](BaseAspect *aspect) -> BaseAspect * {
+        aspect->setParent(dialog);
+        return aspect;
+    };
+
     dialog->setWindowTitle(Tr::tr("Kit CMake Configuration"));
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->setModal(true);
@@ -740,11 +747,10 @@ void CMakeBuildSettingsWidget::kitCMakeConfiguration()
     auto buttons = new QDialogButtonBox(QDialogButtonBox::Close);
     connect(buttons, &QDialogButtonBox::clicked, dialog, &QDialog::close);
 
-    using namespace Layouting;
     Grid {
-        CMakeKitAspect::createKitAspect(kit),
-        CMakeGeneratorKitAspect::createKitAspect(kit),
-        CMakeConfigurationKitAspect::createKitAspect(kit),
+        deleteLater(CMakeKitAspect::createKitAspect(kit)),
+        deleteLater(CMakeGeneratorKitAspect::createKitAspect(kit)),
+        deleteLater(CMakeConfigurationKitAspect::createKitAspect(kit)),
         empty, empty, buttons,
         columnStretch(1, 1)
     }.attachTo(dialog);
@@ -757,19 +763,19 @@ void CMakeBuildSettingsWidget::kitCMakeConfiguration()
 void CMakeBuildSettingsWidget::updateConfigureDetailsWidgetsSummary(
     const QStringList &configurationArguments)
 {
-    ProjectExplorer::ProcessParameters params;
+    FilePath cmakeExecutable = CMakeKitAspect::cmakeExecutable(m_buildConfig->kit());
+    if (cmakeExecutable.isEmpty())
+        cmakeExecutable = "cmake";
 
     CommandLine cmd;
-    const CMakeTool *tool = CMakeKitAspect::cmakeTool(m_buildConfig->kit());
-    cmd.setExecutable(tool ? tool->cmakeExecutable() : "cmake");
-
-    const FilePath buildDirectory = m_buildConfig->buildDirectory();
-
+    cmd.setExecutable(cmakeExecutable);
     cmd.addArgs({"-S", m_buildConfig->project()->projectDirectory().path()});
-    cmd.addArgs({"-B", buildDirectory.path()});
+    cmd.addArgs({"-B", m_buildConfig->buildDirectory().path()});
     cmd.addArgs(configurationArguments);
 
+    ProcessParameters params;
     params.setCommandLine(cmd);
+
     m_configureDetailsWidget->setSummaryText(params.summary(Tr::tr("Configure")));
     m_configureDetailsWidget->setState(DetailsWidget::Expanded);
 }
@@ -788,6 +794,7 @@ void CMakeBuildSettingsWidget::setWarning(const QString &message)
 
 void CMakeBuildSettingsWidget::updateButtonState()
 {
+    QTC_ASSERT(m_buildConfig, return);
     const bool isParsing = m_buildConfig->cmakeBuildSystem()->isParsing();
 
     // Update extra data in buildconfiguration
@@ -1096,10 +1103,8 @@ bool CMakeBuildSettingsWidget::eventFilter(QObject *target, QEvent *event)
     menu->addAction(help);
     connect(help, &QAction::triggered, this, [this, idx] {
         const CMakeConfigItem item = ConfigModel::dataItemFromIndex(idx).toCMakeConfigItem();
-
-        const CMakeTool *tool = CMakeKitAspect::cmakeTool(m_buildConfig->kit());
         const QString linkUrl = "%1/variable/" + QString::fromUtf8(item.key) + ".html";
-        CMakeTool::openCMakeHelpUrl(tool, linkUrl);
+        CMakeKitAspect::openCMakeHelpUrl(m_buildConfig->kit(), linkUrl);
     });
 
     menu->addSeparator();
@@ -1191,10 +1196,10 @@ static CommandLine defaultInitialCMakeCommand(
     const Kit *k, Project *project, const QString &buildType)
 {
     // Generator:
-    CMakeTool *tool = CMakeKitAspect::cmakeTool(k);
-    QTC_ASSERT(tool, return {});
+    const FilePath cmakeExecutable = CMakeKitAspect::cmakeExecutable(k);
+    QTC_ASSERT(!cmakeExecutable.isEmpty(), return {});
 
-    CommandLine cmd{tool->cmakeExecutable()};
+    CommandLine cmd{cmakeExecutable};
     cmd.addArgs(CMakeGeneratorKitAspect::generatorArguments(k));
 
     // CMAKE_BUILD_TYPE:
@@ -1518,8 +1523,6 @@ CMakeBuildConfiguration::CMakeBuildConfiguration(Target *target, Id id)
                                           return QLatin1String();
                                       });
 
-    qmlDebugging.setBuildConfiguration(this);
-
     setInitialBuildAndCleanSteps();
 
     setInitializer([this](const BuildInfo &info) {
@@ -1849,7 +1852,7 @@ void CMakeBuildSystem::setError(const QString &message)
         qCDebug(cmakeBuildConfigurationLog) << "Emitting enabledChanged signal";
         emit buildConfiguration()->enabledChanged();
     }
-    TaskHub::addTask(BuildSystemTask(Task::TaskType::Error, message));
+    TaskHub::addTask<BuildSystemTask>(Task::TaskType::Error, message);
     emit errorOccurred(m_error);
 }
 
@@ -1858,7 +1861,7 @@ void CMakeBuildSystem::setWarning(const QString &message)
     if (m_warning == message)
         return;
     m_warning = message;
-    TaskHub::addTask(BuildSystemTask(Task::TaskType::Warning, message));
+    TaskHub::addTask<BuildSystemTask>(Task::TaskType::Warning, message);
     emit warningOccurred(m_warning);
 }
 
@@ -1878,12 +1881,6 @@ QWidget *CMakeBuildConfiguration::createConfigWidget()
     return m_configWidget;
 }
 
-void CMakeBuildConfiguration::updateInitialCMakeArguments()
-{
-    Q_ASSERT(m_configWidget);
-    m_configWidget->updateInitialCMakeArguments(true);
-}
-
 QStringList CMakeBuildConfiguration::initialCMakeOptions() const
 {
     return initialCMakeArguments.allValues();
@@ -1907,7 +1904,8 @@ QStringList CMakeBuildConfiguration::additionalArgs() const
 void CMakeBuildConfiguration::reconfigure()
 {
     cmakeBuildSystem()->clearCMakeCache();
-    updateInitialCMakeArguments();
+    if (QTC_GUARD(m_configWidget))
+        m_configWidget->updateInitialCMakeArguments();
     cmakeBuildSystem()->runCMake();
 }
 
@@ -2099,12 +2097,12 @@ BuildInfo CMakeBuildConfigurationFactory::createBuildInfo(BuildType buildType)
     switch (buildType) {
     case BuildTypeNone:
         info.typeName = "Build";
-        info.displayName = ::ProjectExplorer::Tr::tr("Build");
+        info.displayName = msgBuildConfigurationBuild();
         info.buildType = BuildConfiguration::Unknown;
         break;
     case BuildTypeDebug: {
         info.typeName = "Debug";
-        info.displayName = ::ProjectExplorer::Tr::tr("Debug");
+        info.displayName = msgBuildConfigurationDebug();
         info.buildType = BuildConfiguration::Debug;
         Store extraInfo;
         // enable QML debugging by default
@@ -2114,7 +2112,7 @@ BuildInfo CMakeBuildConfigurationFactory::createBuildInfo(BuildType buildType)
     }
     case BuildTypeRelease:
         info.typeName = "Release";
-        info.displayName = ::ProjectExplorer::Tr::tr("Release");
+        info.displayName = msgBuildConfigurationRelease();
         info.buildType = BuildConfiguration::Release;
         break;
     case BuildTypeMinSizeRel:
@@ -2129,7 +2127,7 @@ BuildInfo CMakeBuildConfigurationFactory::createBuildInfo(BuildType buildType)
         break;
     case BuildTypeProfile: {
         info.typeName = "Profile";
-        info.displayName = Tr::tr("Profile");
+        info.displayName = msgBuildConfigurationProfile();
         info.buildType = BuildConfiguration::Profile;
         Store extraInfo;
         // override CMake build type, which defaults to info.typeName
@@ -2171,7 +2169,7 @@ CMakeBuildSystem *CMakeBuildConfiguration::cmakeBuildSystem() const
     return qobject_cast<CMakeBuildSystem *>(buildSystem());
 }
 
-void CMakeBuildConfiguration::addToEnvironment(Utils::Environment &env) const
+void CMakeBuildConfiguration::addToEnvironment(Environment &env) const
 {
     // Use the user provided VCPKG_ROOT if existing
     // Visual C++ 2022 (and newer) come with their own VCPKG_ROOT
@@ -2180,9 +2178,9 @@ void CMakeBuildConfiguration::addToEnvironment(Utils::Environment &env) const
     if (!vcpkgRoot.isEmpty())
         env.set(Constants::VCPKG_ROOT, vcpkgRoot);
 
-    const CMakeTool *tool = CMakeKitAspect::cmakeTool(kit());
+    const FilePath cmakeExecutable = CMakeKitAspect::cmakeExecutable(kit());
     // The hack further down is only relevant for desktop
-    if (tool && !tool->cmakeExecutable().isLocal())
+    if (!cmakeExecutable.isEmpty() && !cmakeExecutable.isLocal())
         return;
 
     const FilePath ninja = settings(nullptr).ninjaPath();
