@@ -38,7 +38,8 @@
 #include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
 
-#include <QtTest>
+#include <QElapsedTimer>
+#include <QTest>
 
 using namespace ProjectExplorer;
 using namespace Utils;
@@ -47,11 +48,8 @@ namespace CppEditor::Internal::Tests {
 
 bool isClangFormatPresent()
 {
-    using namespace ExtensionSystem;
-    return Utils::contains(PluginManager::plugins(), [](const PluginSpec *plugin) {
-        return plugin->id() == "clangformat" && plugin->isEffectivelyEnabled();
-    });
-};
+    return ExtensionSystem::PluginManager::specExistsAndIsEnabled("clangformat");
+}
 
 CppTestDocument::CppTestDocument(const QByteArray &fileName, const QByteArray &source,
                                          char cursorMarker)
@@ -464,28 +462,21 @@ FilePath TemporaryDir::createFile(const QByteArray &relativePath, const QByteArr
     return filePath;
 }
 
-static bool copyRecursively(const QString &sourceDirPath,
-                            const QString &targetDirPath,
-                            QString *error)
+static Result<FileUtils::CopyResult> copyHelper(const FilePath &sourcePath,
+                                                const FilePath &targetPath)
 {
-    auto copyHelper = [](const FilePath &sourcePath, const FilePath &targetPath, QString *error) -> bool {
-        if (!sourcePath.copyFile(targetPath)) {
-            if (error) {
-                *error = QString::fromLatin1("copyRecursively() failed: \"%1\" to \"%2\".")
-                            .arg(sourcePath.toUserOutput(), targetPath.toUserOutput());
-            }
-            return false;
-        }
+    if (!sourcePath.copyFile(targetPath)) {
+        return ResultError(QString("copyRecursively() failed: \"%1\" to \"%2\".")
+                               .arg(sourcePath.toUserOutput(), targetPath.toUserOutput()));
+    }
 
-        // Copied files from Qt resources are read-only. Make them writable
-        // so that their parent directory can be removed without warnings.
-        return targetPath.setPermissions(targetPath.permissions() | QFile::WriteUser);
-    };
+    // Copied files from Qt resources are read-only. Make them writable
+    // so that their parent directory can be removed without warnings.
+    Result<> res = targetPath.setPermissions(targetPath.permissions() | QFile::WriteUser);
+    if (!res)
+        return ResultError(res.error());
 
-    return Utils::FileUtils::copyRecursively(Utils::FilePath::fromString(sourceDirPath),
-                                             Utils::FilePath::fromString(targetDirPath),
-                                             error,
-                                             copyHelper);
+    return FileUtils::CopyResult::Done;
 }
 
 TemporaryCopiedDir::TemporaryCopiedDir(const QString &sourceDirPath)
@@ -502,9 +493,11 @@ TemporaryCopiedDir::TemporaryCopiedDir(const QString &sourceDirPath)
         return;
     }
 
-    QString errorMessage;
-    if (!copyRecursively(sourceDirPath, path(), &errorMessage)) {
-        qWarning() << qPrintable(errorMessage);
+    const Result<FileUtils::CopyResult> res =
+        FileUtils::copyRecursively(FilePath::fromString(sourceDirPath), filePath(), copyHelper);
+
+    if (!res) {
+        qWarning() << qPrintable(res.error());
         m_isValid = false;
     }
 }
@@ -527,16 +520,17 @@ int clangdIndexingTimeout()
 SourceFilesRefreshGuard::SourceFilesRefreshGuard()
 {
     connect(CppModelManager::instance(), &CppModelManager::sourceFilesRefreshed, this, [this] {
-        m_refreshed = true;
+        --m_missing;
     });
 }
 
 bool SourceFilesRefreshGuard::wait()
 {
-    for (int i = 0; i < 10 && !m_refreshed; ++i) {
+    for (int i = 0; i < 10 && m_missing > 0; ++i) {
         CppEditor::Tests::waitForSignalOrTimeout(
             CppModelManager::instance(), &CppModelManager::sourceFilesRefreshed, 1000);
     }
-    return m_refreshed;
+    QTC_ASSERT(m_missing >= 0, return true);
+    return m_missing == 0;
 }
 } // namespace CppEditor::Tests
