@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -12,8 +12,8 @@ class RegisterPostMortemAction : public Utils::BoolAspect
 {
 public:
     RegisterPostMortemAction();
-    void readSettings(const QSettings *settings = nullptr) override;
-    void writeSettings(QSettings *) const override {}
+    void readSettings() override;
+    void writeSettings() const override {}
 
 private:
     void registerNow(bool value);
