@@ -1,59 +1,37 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "formwindowfile.h"
+#include "qtcreatorintegration.h"
 #include "designerconstants.h"
 #include "resourcehandler.h"
 
-#include <utils/fileutils.h>
+#include <utils/filepath.h>
+#include <utils/mimeconstants.h>
 #include <utils/qtcassert.h>
 
 #include <QApplication>
 #include <QBuffer>
+#include <QDebug>
 #include <QDesignerFormWindowInterface>
 #include <QDesignerFormWindowManagerInterface>
 #include <QDesignerFormEditorInterface>
 #include <QTextDocument>
 #include <QUndoStack>
-#include <QFileInfo>
-#include <QDebug>
-#include <QTextCodec>
 
+using namespace Core;
 using namespace Utils;
 
-namespace Designer {
-namespace Internal {
+namespace Designer::Internal {
 
 FormWindowFile::FormWindowFile(QDesignerFormWindowInterface *form, QObject *parent)
   : m_formWindow(form)
 {
-    setMimeType(Designer::Constants::FORM_MIMETYPE);
+    setMimeType(Utils::Constants::FORM_MIMETYPE);
     setParent(parent);
     setId(Utils::Id(Designer::Constants::K_DESIGNER_XML_EDITOR_ID));
     // Designer needs UTF-8 regardless of settings.
-    setCodec(QTextCodec::codecForName("UTF-8"));
+    setEncoding(TextEncoding::Utf8);
     connect(m_formWindow->core()->formWindowManager(), &QDesignerFormWindowManagerInterface::formWindowRemoved,
             this, &FormWindowFile::slotFormWindowRemoved);
     connect(m_formWindow->commandHistory(), &QUndoStack::indexChanged,
@@ -65,75 +43,65 @@ FormWindowFile::FormWindowFile(QDesignerFormWindowInterface *form, QObject *pare
             m_resourceHandler, &ResourceHandler::updateResources);
 }
 
-Core::IDocument::OpenResult FormWindowFile::open(QString *errorString, const QString &fileName,
-                                                 const QString &realFileName)
+Result<> FormWindowFile::open(const FilePath &filePath, const FilePath &realFilePath)
 {
     if (Designer::Constants::Internal::debug)
-        qDebug() << "FormWindowFile::open" << fileName;
+        qDebug() << "FormWindowFile::open" << filePath.toUserOutput();
 
     QDesignerFormWindowInterface *form = formWindow();
-    QTC_ASSERT(form, return OpenResult::CannotHandle);
+    QTC_ASSERT(form, return ResultError(ResultAssert));
 
-    if (fileName.isEmpty())
-        return OpenResult::ReadError;
+    if (filePath.isEmpty())
+        return ResultError("File name is empty"); // FIXME: Use something better
 
-    const QFileInfo fi(fileName);
-    const QString absfileName = fi.absoluteFilePath();
+    TextFileFormat::ReadResult readResult = read(filePath.absoluteFilePath());
+    if (readResult.code != TextFileFormat::ReadSuccess)
+        return ResultError(readResult.error);
 
-    QString contents;
-    Utils::TextFileFormat::ReadResult readResult = read(absfileName, &contents, errorString);
-    if (readResult == Utils::TextFileFormat::ReadEncodingError)
-        return OpenResult::CannotHandle;
-    if (readResult != Utils::TextFileFormat::ReadSuccess)
-        return OpenResult::ReadError;
-
-    form->setFileName(absfileName);
-    const QByteArray contentsBA = contents.toUtf8();
+    form->setFileName(filePath.absoluteFilePath().toUrlishString());
+    const QByteArray contentsBA = readResult.content.toUtf8();
     QBuffer str;
     str.setData(contentsBA);
     str.open(QIODevice::ReadOnly);
-    if (!form->setContents(&str, errorString))
-        return OpenResult::CannotHandle;
-    form->setDirty(fileName != realFileName);
+    QString errorString;
+    if (!form->setContents(&str, &errorString))
+        return ResultError(errorString);
+    form->setDirty(filePath != realFilePath);
 
     syncXmlFromFormWindow();
-    setFilePath(Utils::FilePath::fromString(absfileName));
+    setFilePath(filePath.absoluteFilePath());
     setShouldAutoSave(false);
     resourceHandler()->updateProjectResources();
 
-    return OpenResult::Success;
+    return ResultOk;
 }
 
-bool FormWindowFile::save(QString *errorString, const QString &name, bool autoSave)
+Result<> FormWindowFile::saveImpl(const FilePath &filePath, SaveOption option)
 {
-    const FilePath actualName = name.isEmpty() ? filePath() : FilePath::fromString(name);
-
-    if (Designer::Constants::Internal::debug)
-        qDebug() << Q_FUNC_INFO << name << "->" << actualName;
-
-    QTC_ASSERT(m_formWindow, return false);
-
-    if (actualName.isEmpty())
-        return false;
+    if (!m_formWindow)
+        return ResultError("ASSERT: FormWindoFile: !m_formWindow");
+    if (filePath.isEmpty())
+        return ResultError("ASSERT: FormWindowFile: filePath.isEmpty()");
 
     const QString oldFormName = m_formWindow->fileName();
-    if (!autoSave)
-        m_formWindow->setFileName(actualName.toString());
-    const bool writeOK = writeFile(actualName.toString(), errorString);
-    m_shouldAutoSave = false;
-    if (autoSave)
-        return writeOK;
+    if (option != SaveOption::AutoSave)
+        m_formWindow->setFileName(filePath.toUrlishString());
 
-    if (!writeOK) {
+    const Result<> res = writeFile(filePath);
+    m_shouldAutoSave = false;
+    if (option == SaveOption::AutoSave)
+        return res;
+
+    if (!res) {
         m_formWindow->setFileName(oldFormName);
-        return false;
+        return res;
     }
 
     m_formWindow->setDirty(false);
-    setFilePath(actualName);
+    setFilePath(filePath);
     updateIsModified();
 
-    return true;
+    return ResultOk;
 }
 
 QByteArray FormWindowFile::contents() const
@@ -141,17 +109,17 @@ QByteArray FormWindowFile::contents() const
     return formWindowContents().toUtf8();
 }
 
-bool FormWindowFile::setContents(const QByteArray &contents)
+Result<> FormWindowFile::setContents(const QByteArray &contents)
 {
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << contents.size();
 
     document()->clear();
 
-    QTC_ASSERT(m_formWindow, return false);
+    QTC_ASSERT(m_formWindow, return ResultError(ResultAssert));
 
     if (contents.isEmpty())
-        return false;
+        return ResultError(ResultAssert);
 
     // If we have an override cursor, reset it over Designer loading,
     // should it pop up messages about missing resources or such.
@@ -168,16 +136,16 @@ bool FormWindowFile::setContents(const QByteArray &contents)
         QApplication::setOverrideCursor(overrideCursor);
 
     if (!success)
-        return false;
+        return ResultError(ResultAssert);
 
     syncXmlFromFormWindow();
     setShouldAutoSave(false);
-    return true;
+    return ResultOk;
 }
 
 void FormWindowFile::setFilePath(const FilePath &newName)
 {
-    m_formWindow->setFileName(newName.toString());
+    m_formWindow->setFileName(newName.toUrlishString());
     IDocument::setFilePath(newName);
 }
 
@@ -210,11 +178,11 @@ bool FormWindowFile::isSaveAsAllowed() const
     return true;
 }
 
-bool FormWindowFile::reload(QString *errorString, ReloadFlag flag, ChangeType type)
+Result<> FormWindowFile::reload(ReloadFlag flag, ChangeType type)
 {
     if (flag == FlagIgnore) {
         if (!m_formWindow || type != TypeContents)
-            return true;
+            return ResultOk;
         const bool wasModified = m_formWindow->isDirty();
         {
             Utils::GuardLocker locker(m_modificationChangedGuard);
@@ -224,18 +192,13 @@ bool FormWindowFile::reload(QString *errorString, ReloadFlag flag, ChangeType ty
         }
         if (!wasModified)
             updateIsModified();
-        return true;
-    }
-    if (type == TypePermissions) {
-        emit changed();
+        return ResultOk;
     } else {
         emit aboutToReload();
-        const bool success
-                = (open(errorString, filePath().toString(), filePath().toString()) == OpenResult::Success);
-        emit reloadFinished(success);
-        return success;
+        const Result<> result = open(filePath(), filePath());
+        emit reloadFinished(result.has_value());
+        return result;
     }
-    return true;
 }
 
 void FormWindowFile::setFallbackSaveAsFileName(const QString &fn)
@@ -251,11 +214,20 @@ QString FormWindowFile::fallbackSaveAsFileName() const
     return m_suggestedName;
 }
 
-bool FormWindowFile::writeFile(const QString &fn, QString *errorString) const
+bool FormWindowFile::supportsEncoding(const TextEncoding &encoding) const
+{
+    return encoding.isUtf8();
+}
+
+Result<> FormWindowFile::writeFile(const FilePath &filePath) const
 {
     if (Designer::Constants::Internal::debug)
-        qDebug() << Q_FUNC_INFO << filePath() << fn;
-    return write(fn, format(), m_formWindow->contents(), errorString);
+        qDebug() << Q_FUNC_INFO << this->filePath() << filePath;
+    auto *integration = qobject_cast<QtCreatorIntegration *>(m_formWindow->core()->integration());
+    Q_ASSERT(integration);
+    if (!integration->setQtVersionFromFile(filePath))
+        integration->resetQtVersion();
+    return write(filePath, format(), m_formWindow->contents());
 }
 
 QDesignerFormWindowInterface *FormWindowFile::formWindow() const
@@ -289,5 +261,4 @@ void FormWindowFile::slotFormWindowRemoved(QDesignerFormWindowInterface *w)
         m_formWindow = nullptr;
 }
 
-} // namespace Internal
-} // namespace Designer
+} // namespace Designer::Internal
