@@ -10,6 +10,7 @@
 #include "project.h"
 #include "projectexplorer.h"
 #include "projectexplorerconstants.h"
+#include "projectexplorersettings.h"
 #include "projectexplorertr.h"
 #include "projectmanager.h"
 #include "projectnodes.h"
@@ -31,8 +32,9 @@
 #include <utils/stringutils.h>
 #include <utils/variablechooser.h>
 
-#include <QComboBox>
+#include <QCheckBox>
 #include <QHash>
+#include <QLabel>
 #include <QLayout>
 #include <QPushButton>
 #include <QLoggingCategory>
@@ -44,11 +46,11 @@ namespace ProjectExplorer {
 
 const char BUILD_KEY[] = "ProjectExplorer.RunConfiguration.BuildKey";
 const char CUSTOMIZED_KEY[] = "ProjectExplorer.RunConfiguration.Customized";
-
+const char UNIQUE_ID_KEY[] = "ProjectExplorer.RunConfiguration.UniqueId";
 
 ///////////////////////////////////////////////////////////////////////
 //
-// IRunConfigurationAspect
+// GlobalOrProjectAspect
 //
 ///////////////////////////////////////////////////////////////////////
 
@@ -68,15 +70,19 @@ void GlobalOrProjectAspect::setProjectSettings(AspectContainer *settings)
     m_projectSettings->setAutoApply(true);
 }
 
-void GlobalOrProjectAspect::setGlobalSettings(AspectContainer *settings)
+void GlobalOrProjectAspect::setGlobalSettings(AspectContainer *settings, Id settingsPage)
 {
     m_globalSettings = settings;
     m_globalSettings->setAutoApply(false);
+    m_settingsPage = settingsPage;
 }
 
 void GlobalOrProjectAspect::setUsingGlobalSettings(bool value)
 {
-    m_useGlobalSettings = value;
+    if (m_useGlobalSettings != value) {
+        m_useGlobalSettings = value;
+        emit currentSettingsChanged();
+    }
 }
 
 AspectContainer *GlobalOrProjectAspect::currentSettings() const
@@ -112,13 +118,65 @@ void GlobalOrProjectAspect::toActiveMap(Store &data) const
 void GlobalOrProjectAspect::resetProjectToGlobalSettings()
 {
     QTC_ASSERT(m_globalSettings, return);
+    if (!m_projectSettings)
+        return;
     Store map;
     m_globalSettings->toMap(map);
-    if (m_projectSettings)
-        m_projectSettings->fromMap(map);
+    m_projectSettings->fromMap(map);
+    emit wasResetToGlobalValues();
 }
 
 
+class GlobalOrProjectAspectWidget : public QWidget
+{
+public:
+    explicit GlobalOrProjectAspectWidget(GlobalOrProjectAspect *aspect)
+    {
+        using namespace Layouting;
+
+        const auto useGlobalCheckBox = new QCheckBox;
+        const auto useGlobalLabel = new QLabel;
+        if (aspect->settingsPage().isValid()) {
+            useGlobalLabel->setText(Tr::tr("Use <a href=\"dummy\">global settings</a>"));
+            connect(useGlobalLabel, &QLabel::linkActivated, aspect, [aspect] {
+                Core::ICore::showOptionsDialog(aspect->settingsPage());
+            });
+        } else {
+            useGlobalLabel->setText(Tr::tr("Use global settings"));
+        }
+
+        auto restoreButton = new QPushButton(Tr::tr("Restore Global"));
+
+        auto innerPane = new QWidget;
+        auto configWidget = aspect->projectSettings()->layouter()().emerge();
+
+        Column {
+            Row { useGlobalCheckBox, useGlobalLabel, st },
+            hr,
+            Row { restoreButton, st },
+            configWidget
+        }.attachTo(innerPane);
+
+        Column { innerPane }.attachTo(this);
+
+        innerPane->layout()->setContentsMargins(0, 0, 0, 0);
+        layout()->setContentsMargins(0, 0, 0, 0);
+
+        auto chooseSettings = [=](bool isGlobal) {
+            useGlobalCheckBox->setChecked(isGlobal);
+            aspect->setUsingGlobalSettings(isGlobal);
+            configWidget->setEnabled(!isGlobal);
+            restoreButton->setEnabled(!isGlobal);
+        };
+
+        chooseSettings(aspect->isUsingGlobalSettings());
+
+        connect(useGlobalCheckBox, &QCheckBox::clicked, this, chooseSettings);
+        connect(restoreButton, &QPushButton::clicked,
+                aspect, &ProjectExplorer::GlobalOrProjectAspect::resetProjectToGlobalSettings);
+    }
+};
+
 class RunConfigAspectWidget : public QWidget
 {
 public:
@@ -126,49 +184,29 @@ public:
     {
         using namespace Layouting;
 
-        auto settingsCombo = new QComboBox;
-        settingsCombo->addItem(Tr::tr("Global"));
-        settingsCombo->addItem(Tr::tr("Custom"));
-
-        auto restoreButton = new QPushButton(Tr::tr("Restore Global"));
-
-        auto innerPane = new QWidget;
-        auto configWidget = aspect->projectSettings()->layouter()().emerge();
-
         auto details = new DetailsWidget;
-        details->setWidget(innerPane);
-
-        Column {
-            Row { settingsCombo, restoreButton, st },
-            configWidget
-        }.attachTo(innerPane);
+        details->setWidget(createGlobalOrProjectAspectWidget(aspect));
 
         Column { details }.attachTo(this);
 
         details->layout()->setContentsMargins(0, 0, 0, 0);
-        innerPane->layout()->setContentsMargins(0, 0, 0, 0);
         layout()->setContentsMargins(0, 0, 0, 0);
 
-        auto chooseSettings = [=](int setting) {
-            const bool isCustom = (setting == 1);
-
-            settingsCombo->setCurrentIndex(setting);
-            aspect->setUsingGlobalSettings(!isCustom);
-            configWidget->setEnabled(isCustom);
-            restoreButton->setEnabled(isCustom);
-            details->setSummaryText(isCustom
-                                    ? Tr::tr("Use Customized Settings")
-                                    : Tr::tr("Use Global Settings"));
+        const auto updateDetails = [details, aspect] {
+            details->setSummaryText(aspect->isUsingGlobalSettings()
+                                    ? Tr::tr("Use Global Settings")
+                                    : Tr::tr("Use Customized Settings"));
         };
-
-        chooseSettings(aspect->isUsingGlobalSettings() ? 0 : 1);
-
-        connect(settingsCombo, &QComboBox::activated, this, chooseSettings);
-        connect(restoreButton, &QPushButton::clicked,
-                aspect, &ProjectExplorer::GlobalOrProjectAspect::resetProjectToGlobalSettings);
+        connect(aspect, &GlobalOrProjectAspect::currentSettingsChanged, this, updateDetails);
+        updateDetails();
     }
 };
 
+QWidget *createGlobalOrProjectAspectWidget(GlobalOrProjectAspect *aspect)
+{
+    return new GlobalOrProjectAspectWidget(aspect);
+}
+
 QWidget *createRunConfigAspectWidget(GlobalOrProjectAspect *aspect)
 {
     return new RunConfigAspectWidget(aspect);
@@ -198,7 +236,7 @@ static std::vector<RunConfiguration::AspectFactory> theAspectFactories;
 
 static QList<RunConfigurationFactory *> g_runConfigurationFactories;
 
-RunConfiguration::RunConfiguration(BuildConfiguration *bc, Utils::Id id)
+RunConfiguration::RunConfiguration(BuildConfiguration *bc, Id id)
     : ProjectConfiguration(bc->target(), id), m_buildConfiguration(bc)
 {
     forceDisplayNameSerialization();
@@ -207,24 +245,14 @@ RunConfiguration::RunConfiguration(BuildConfiguration *bc, Utils::Id id)
     MacroExpander &expander = *macroExpander();
     expander.setDisplayName(Tr::tr("Run Settings"));
     expander.setAccumulating(true);
-    expander.registerSubProvider([bc] { return bc->macroExpander(); });
-    expander.registerPrefix("RunConfig:Env", Tr::tr("Variables in the run environment."),
-                             [this](const QString &var) {
-        const auto envAspect = aspect<EnvironmentAspect>();
-        return envAspect ? envAspect->environment().expandedValueForKey(var) : QString();
-    });
+    expander.registerSubProvider({this, [bc] { return bc->macroExpander(); }});
+    setupMacroExpander(expander, this, false);
     expander.registerVariable("RunConfig:WorkingDir",
                                Tr::tr("The run configuration's working directory."),
                                [this] {
         const auto wdAspect = aspect<WorkingDirectoryAspect>();
         return wdAspect ? wdAspect->workingDirectory().toUrlishString() : QString();
     });
-    expander.registerVariable("RunConfig:Name", Tr::tr("The run configuration's name."),
-            [this] { return displayName(); });
-    expander.registerFileVariables("RunConfig:Executable",
-                                     Tr::tr("The run configuration's executable."),
-                                     [this] { return commandLine().executable(); });
-
 
     m_commandLineGetter = [this] {
         Launcher launcher;
@@ -251,16 +279,30 @@ RunConfiguration::RunConfiguration(BuildConfiguration *bc, Utils::Id id)
     connect(bc->buildSystem(), &BuildSystem::deploymentDataChanged,
             this, &RunConfiguration::update);
     connect(bc, &BuildConfiguration::kitChanged, this, &RunConfiguration::update);
+
+    connect(this, &AspectContainer::subAspectChanged, this, [this](BaseAspect *aspect) {
+        if (!buildKey().isEmpty() && qobject_cast<ExecutableAspect *>(aspect))
+            return;
+        forEachLinkedRunConfig([aspect](RunConfiguration *rc) {
+            // We assume that no run configuration has more than one aspect
+            // of the same type.
+            if (BaseAspect * const other = rc->aspect(aspect->id())) {
+                Store map;
+                aspect->toMap(map);
+                other->fromMap(map);
+            }
+        });
+    });
 }
 
 RunConfiguration::~RunConfiguration() = default;
 
-QString RunConfiguration::disabledReason(Utils::Id) const
+QString RunConfiguration::disabledReason(Id) const
 {
     return buildSystem()->disabledReason(m_buildKey);
 }
 
-bool RunConfiguration::isEnabled(Utils::Id) const
+bool RunConfiguration::isEnabled(Id) const
 {
     return buildSystem()->hasParsingData();
 }
@@ -277,9 +319,9 @@ QWidget *RunConfiguration::createConfigurationWidget()
     }
     auto widget = form.emerge();
 
-    VariableChooser::addSupportForChildWidgets(widget, macroExpander());
+    VariableChooser::addSupportForChildWidgets(widget, {this, macroExpander()});
 
-    auto detailsWidget = new Utils::DetailsWidget;
+    auto detailsWidget = new DetailsWidget;
     detailsWidget->setState(DetailsWidget::NoSummary);
     detailsWidget->setWidget(widget);
     return detailsWidget;
@@ -287,7 +329,7 @@ QWidget *RunConfiguration::createConfigurationWidget()
 
 bool RunConfiguration::isConfigured() const
 {
-    return !Utils::anyOf(checkForIssues(), [](const Task &t) { return t.type == Task::Error; });
+    return !Utils::anyOf(checkForIssues(), &Task::isError);
 }
 
 bool RunConfiguration::isCustomized() const
@@ -349,6 +391,51 @@ BuildSystem *RunConfiguration::buildSystem() const
     return m_buildConfiguration->buildSystem();
 }
 
+bool RunConfiguration::equals(const RunConfiguration *other) const
+{
+    Store map;
+    toMapSimple(map);
+    Store otherMap;
+    other->toMapSimple(otherMap);
+    return map == otherMap;
+}
+
+void RunConfiguration::setupMacroExpander(
+    MacroExpander &exp, const RunConfiguration *rc, bool documentationOnly)
+{
+    exp.registerPrefix(
+        "RunConfig:Env",
+        "USER",
+        Tr::tr("Variables in the run environment."),
+        [rc](const QString &var) {
+            if (!rc)
+                return QString();
+            const auto envAspect = rc->aspect<EnvironmentAspect>();
+            return envAspect ? envAspect->environment().expandedValueForKey(var) : QString();
+        },
+        true,
+        !documentationOnly);
+
+    exp.registerVariable("RunConfig:Name", Tr::tr("The run configuration's name."), [rc] {
+            return rc ? rc->displayName() : QString();
+        }, true, !documentationOnly);
+
+    exp.registerFileVariables(
+        "RunConfig:Executable", Tr::tr("The run configuration's executable."), [rc] {
+            return rc ? rc->commandLine().executable() : FilePath();
+        }, true, !documentationOnly);
+}
+
+void RunConfiguration::setExecutionType(Id executionType)
+{
+    m_executionType = executionType;
+}
+
+Id RunConfiguration::executionType() const
+{
+    return m_executionType;
+}
+
 void RunConfiguration::setUpdater(const Updater &updater)
 {
     m_updater = updater;
@@ -376,6 +463,7 @@ void RunConfiguration::toMapSimple(Store &map) const
     }
 
     map.insert(BUILD_KEY, m_buildKey);
+    map.insert(UNIQUE_ID_KEY, m_uniqueId);
 }
 
 void RunConfiguration::setCommandLineGetter(const CommandLineGetter &cmdGetter)
@@ -431,6 +519,42 @@ void RunConfiguration::cloneFromOther(const RunConfiguration *rc)
     fromMap(copyData);
 }
 
+const QList<BuildConfiguration *> RunConfiguration::syncableBuildConfigurations() const
+{
+    QList<BuildConfiguration *> buildConfigs;
+    switch (ProjectExplorerSettings::get(this).syncRunConfigurations.value()) {
+    case SyncRunConfigs::Off:
+        break;
+    case SyncRunConfigs::SameKit:
+        buildConfigs = target()->buildConfigurations();
+        break;
+    case SyncRunConfigs::All:
+        buildConfigs = project()->allBuildConfigurations();
+        break;
+    }
+    buildConfigs.removeOne(buildConfiguration());
+    return buildConfigs;
+}
+
+void RunConfiguration::forEachLinkedRunConfig(const std::function<void(RunConfiguration *)> &handler)
+{
+    for (BuildConfiguration * const bc : syncableBuildConfigurations()) {
+        for (RunConfiguration * const rc : bc->runConfigurations()) {
+            if (rc->uniqueId() == uniqueId()) {
+                handler(rc);
+                break;
+            }
+        }
+    }
+}
+
+void RunConfiguration::makeActive()
+{
+    buildConfiguration()->setActiveRunConfiguration(this);
+    forEachLinkedRunConfig(
+        [](RunConfiguration *rc) { rc->buildConfiguration()->setActiveRunConfiguration(rc); });
+}
+
 BuildTargetInfo RunConfiguration::buildTargetInfo() const
 {
     BuildSystem *bs = buildSystem();
@@ -453,6 +577,7 @@ void RunConfiguration::fromMap(const Store &map)
 
     m_customized = m_customized || map.value(CUSTOMIZED_KEY, false).toBool();
     m_buildKey = map.value(BUILD_KEY).toString();
+    m_uniqueId = map.value(UNIQUE_ID_KEY).toString();
 
     if (m_usesEmptyBuildKeys) {
         QTC_CHECK(m_buildKey.isEmpty());
@@ -462,9 +587,9 @@ void RunConfiguration::fromMap(const Store &map)
 }
 
 /*!
-    \class ProjectExplorer::IRunConfigurationAspect
+    \class ProjectExplorer::GlobalOrProjectAspect
 
-    \brief The IRunConfigurationAspect class provides an additional
+    \brief The GlobalOrProjectAspect class provides an additional
     configuration aspect.
 
     Aspects are a mechanism to add RunControl-specific options to a run
@@ -503,7 +628,17 @@ ProcessRunData RunConfiguration::runnable() const
     if (auto workingDirectoryAspect = aspect<WorkingDirectoryAspect>())
         r.workingDirectory = r.command.executable().withNewMappedPath(workingDirectoryAspect->workingDirectory());
     if (auto environmentAspect = aspect<EnvironmentAspect>())
-        r.environment = environmentAspect->environment();
+    {
+        r.environment = environmentAspect->expandedEnvironment(*macroExpander());
+        auto enableCategoriesFilterAspect = aspect<EnableCategoriesFilterAspect>();
+        if (enableCategoriesFilterAspect && enableCategoriesFilterAspect->value()) {
+            r.environment.set("QT_LOGGING_RULES", "_logging_categories=true;*.debug=true");
+            r.environment.set("QT_MESSAGE_PATTERN",
+                              "%{category}:[%{if-debug}D%{endif}%{if-info}I%{endif}"
+                              "%{if-warning}W%{endif}%{if-critical}C%{endif}"
+                              "%{if-fatal}F%{endif}] %{message}");
+        }
+    }
     if (m_runnableModifier)
         m_runnableModifier(r);
 
@@ -571,19 +706,17 @@ RunConfigurationFactory::~RunConfigurationFactory()
 QString RunConfigurationFactory::decoratedTargetName(const QString &targetName, Kit *kit)
 {
     QString displayName = targetName;
-    Utils::Id devType = RunDeviceTypeKitAspect::deviceTypeId(kit);
-    if (devType != Constants::DESKTOP_DEVICE_TYPE) {
-        if (IDevice::ConstPtr dev = RunDeviceKitAspect::device(kit)) {
-            if (displayName.isEmpty()) {
-                //: Shown in Run configuration if no executable is given, %1 is device name
-                displayName = Tr::tr("Run on %{Device:Name}");
-            } else {
-                //: Shown in Run configuration, Add menu: "name of runnable (on device name)"
-                displayName = Tr::tr("%1 (on %{Device:Name})").arg(displayName);
-            }
-        }
-    }
-    return displayName;
+    Id devType = RunDeviceTypeKitAspect::deviceTypeId(kit);
+
+    if (devType == Constants::DESKTOP_DEVICE_TYPE)
+        return displayName;
+
+    //: Shown in Run configuration if no executable is given
+    if (displayName.isEmpty())
+        return Tr::tr("Run on %{Device:Name}");
+
+    //: Shown in Run configuration, Add menu: "name of runnable (on device name)"
+    return Tr::tr("%1 (on %{Device:Name})").arg(displayName);
 }
 
 QList<RunConfigurationCreationInfo>
@@ -633,7 +766,7 @@ bool RunConfigurationFactory::supportsBuildKey(BuildConfiguration *bc, const QSt
     \sa addSupportedProjectType()
 */
 
-void RunConfigurationFactory::addSupportedTargetDeviceType(Utils::Id id)
+void RunConfigurationFactory::addSupportedTargetDeviceType(Id id)
 {
     m_supportedTargetDeviceTypes.append(id);
 }
@@ -655,11 +788,21 @@ void RunConfigurationFactory::setDecorateDisplayNames(bool on)
     \sa addSupportedTargetDeviceType()
 */
 
-void RunConfigurationFactory::addSupportedProjectType(Utils::Id id)
+void RunConfigurationFactory::addSupportedProjectType(Id id)
 {
     m_supportedProjectTypes.append(id);
 }
 
+void RunConfigurationFactory::setExecutionTypeId(Id executionType)
+{
+    m_executionType = executionType;
+}
+
+Id RunConfigurationFactory::executionTypeId() const
+{
+    return m_executionType;
+}
+
 bool RunConfigurationFactory::canHandle(Target *target) const
 {
     const Project *project = target->project();
@@ -668,14 +811,19 @@ bool RunConfigurationFactory::canHandle(Target *target) const
     if (containsType(target->project()->projectIssues(kit), Task::TaskType::Error))
         return false;
 
-    if (!m_supportedProjectTypes.isEmpty())
-        if (!m_supportedProjectTypes.contains(project->id()))
-            return false;
+    const bool supportsAnyExecutionType = !m_executionType.isValid();
+    if (!supportsAnyExecutionType && RunDeviceTypeKitAspect::executionTypeId(kit) != m_executionType)
+        return false;
 
-    if (!m_supportedTargetDeviceTypes.isEmpty())
-        if (!m_supportedTargetDeviceTypes.contains(
-                    RunDeviceTypeKitAspect::deviceTypeId(kit)))
+    if (!m_supportedProjectTypes.isEmpty()) {
+        if (!m_supportedProjectTypes.contains(project->type()))
             return false;
+    }
+
+    if (!m_supportedTargetDeviceTypes.isEmpty()) {
+        if (!m_supportedTargetDeviceTypes.contains(RunDeviceTypeKitAspect::deviceTypeId(kit)))
+            return false;
+    }
 
     return true;
 }
@@ -690,6 +838,8 @@ RunConfiguration *RunConfigurationFactory::create(BuildConfiguration *bc) const
     for (const RunConfiguration::AspectFactory &factory : theAspectFactories)
         rc->registerAspect(factory(bc), true);
 
+    rc->setExecutionType(executionTypeId());
+
     return rc;
 }
 
@@ -704,6 +854,8 @@ RunConfiguration *RunConfigurationCreationInfo::create(BuildConfiguration *bc) c
     rc->m_buildKey = buildKey;
     rc->update();
     rc->setDisplayName(displayName);
+
+    // THIS MUST BE CALLED LAST!
     rc->setPristineState();
 
     return rc;
@@ -713,7 +865,7 @@ RunConfiguration *RunConfigurationFactory::restore(BuildConfiguration *bc, const
 {
     for (RunConfigurationFactory *factory : std::as_const(g_runConfigurationFactories)) {
         if (factory->canHandle(bc->target())) {
-            const Utils::Id id = idFromMap(map);
+            const Id id = idFromMap(map);
             if (id.name().startsWith(factory->m_runConfigurationId.name())) {
                 RunConfiguration *rc = factory->create(bc);
                 rc->fromMap(map);
@@ -789,4 +941,25 @@ RunConfiguration *activeRunConfigForCurrentProject()
     return activeRunConfig(ProjectTree::currentProject());
 }
 
+QString ProjectExplorer::RunConfiguration::uniqueId() const
+{
+    if (!m_uniqueId.isEmpty())
+        return m_uniqueId;
+    return buildKey();
+}
+
+void RunConfiguration::setUniqueId(const QString &id)
+{
+    m_uniqueId = id;
+}
+
+QString RunConfiguration::expandedDisplayName() const
+{
+    QString displayName = ProjectConfiguration::expandedDisplayName();
+    if (hasCreator())
+        return displayName;
+
+    return joinStrings({displayName, QString("[%1]").arg(Tr::tr("unavailable"))}, ' ');
+}
+
 } // namespace ProjectExplorer
