@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -7,21 +7,21 @@
 
 #include <QWidget>
 
-QT_FORWARD_DECLARE_CLASS(QSpinBox)
+QT_BEGIN_NAMESPACE
+class QCheckBox;
+class QComboBox;
+class QLineEdit;
+class QSpinBox;
+QT_END_NAMESPACE
 
 namespace QmlDesigner {
 
-namespace Ui {
-class TimelineAnimationForm;
-}
-
 class TimelineAnimationForm : public QWidget
 {
     Q_OBJECT
 
 public:
     explicit TimelineAnimationForm(QWidget *parent);
-    ~TimelineAnimationForm() override;
     void setup(const ModelNode &animation);
     ModelNode animation() const;
 
@@ -33,7 +33,15 @@ private:
     void connectSpinBox(QSpinBox *spinBox, const PropertyName &propertyName);
     void populateStateComboBox();
 
-    Ui::TimelineAnimationForm *ui;
+    QLineEdit *m_idLineEdit;
+    QCheckBox *m_running;
+    QSpinBox *m_startFrame;
+    QSpinBox *m_endFrame;
+    QSpinBox *m_duration;
+    QCheckBox *m_continuous;
+    QSpinBox *m_loops;
+    QCheckBox *m_pingPong;
+    QComboBox *m_transitionToState;
 
     QmlTimeline m_timeline;
     ModelNode m_animation;
