@@ -1,39 +1,21 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qdsnewdialog.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/iwizardfactory.h>
-#include <utils/qtcassert.h>
+
 #include <utils/algorithm.h>
+#include <utils/fileutils.h>
+#include <utils/qtcassert.h>
+
 #include <qmldesigner/components/componentcore/theme.h>
+#include <qmldesigner/qmldesignerconstants.h>
 
 #include "createproject.h"
-#include "wizardfactories.h"
 #include "newprojectdialogimageprovider.h"
+#include "wizardfactories.h"
 
 #include <QMessageBox>
 #include <QQmlContext>
@@ -67,7 +49,7 @@ QString uniqueProjectName(const QString &path)
 /***********************/
 
 QdsNewDialog::QdsNewDialog(QWidget *parent)
-    : m_dialog{new QQuickWidget(parent)}
+    : m_dialog{Utils::makeUniqueObjectPtr<QQuickWidget>(parent)}
     , m_categoryModel{new PresetCategoryModel(&m_presetData, this)}
     , m_presetModel{new PresetModel(&m_presetData, this)}
     , m_screenSizeModel{new ScreenSizeModel(this)}
@@ -75,18 +57,19 @@ QdsNewDialog::QdsNewDialog(QWidget *parent)
     , m_recentsStore{"RecentPresets.json", StorePolicy::UniqueValues}
     , m_userPresetsStore{"UserPresets.json", StorePolicy::UniqueNames}
 {
-    setParent(m_dialog);
+    connect(m_dialog.get(), &QObject::destroyed, this, &QObject::deleteLater);
 
     m_recentsStore.setReverseOrder();
     m_recentsStore.setMaximum(10);
 
+    m_dialog->setObjectName(QmlDesigner::Constants::OBJECT_NAME_NEW_DIALOG);
     m_dialog->setResizeMode(QQuickWidget::SizeRootObjectToView); // SizeViewToRootObject
     m_dialog->engine()->addImageProvider(QStringLiteral("newprojectdialog_library"),
                                          new Internal::NewProjectDialogImageProvider());
     QmlDesigner::Theme::setupTheme(m_dialog->engine());
     qmlRegisterSingletonInstance<QdsNewDialog>("BackendApi", 1, 0, "BackendApi", this);
-    m_dialog->engine()->addImportPath(Core::ICore::resourcePath("qmldesigner/propertyEditorQmlSources/imports").toString());
-    m_dialog->engine()->addImportPath(Core::ICore::resourcePath("qmldesigner/newprojectdialog/imports").toString());
+    m_dialog->engine()->addImportPath(Core::ICore::resourcePath("qmldesigner/propertyEditorQmlSources/imports").toUrlishString());
+    m_dialog->engine()->addImportPath(Core::ICore::resourcePath("qmldesigner/newprojectdialog/imports").toUrlishString());
     m_dialog->setSource(QUrl::fromLocalFile(qmlPath()));
 
     m_dialog->setWindowModality(Qt::ApplicationModal);
@@ -105,22 +88,28 @@ QdsNewDialog::QdsNewDialog(QWidget *parent)
 
     m_dialog->installEventFilter(this);
 
-    QObject::connect(&m_wizard, &WizardHandler::wizardCreationFailed, this, [this]() {
-        QMessageBox::critical(m_dialog, tr("New project"), tr("Failed to initialize data"));
-        reject();
-        delete this;
-    });
-
-    QObject::connect(m_styleModel.data(), &StyleModel::modelAboutToBeReset, this, [this]() {
-        m_qmlStyleIndex = -1;
+    QObject::connect(&m_wizard, &WizardHandler::wizardCreationFailed, this, [this] {
+        // TODO: if the dialog itself could react on the error this would not be necessary
+        QMessageBox::critical(m_dialog.get(), tr("New Project"), tr("Failed to initialize data."));
+        m_dialog->close();
     });
 }
 
 bool QdsNewDialog::eventFilter(QObject *obj, QEvent *event)
 {
-    if (obj == m_dialog && event->type() == QEvent::KeyPress
+    if (obj != m_dialog.get())
+        return false;
+
+    if (event->type() == QEvent::KeyPress
         && static_cast<QKeyEvent *>(event)->key() == Qt::Key_Escape) {
-        reject();
+        m_dialog->close();
+        return true;
+    }
+
+    if (event->type() == QEvent::Close) {
+        m_screenSizeModel->setBackendModel(nullptr);
+        m_styleModel->setSourceModel(nullptr);
+        m_wizard.destroyWizard();
         return true;
     }
 
@@ -133,8 +122,7 @@ void QdsNewDialog::onDeletingWizard()
     m_qmlScreenSizeIndex = -1;
     m_screenSizeModel->reset();
 
-    m_styleModel->setBackendModel(nullptr);
-    m_qmlStyleIndex = -1;
+    m_styleModel->setSourceModel(nullptr);
 }
 
 void QdsNewDialog::setProjectName(const QString &name)
@@ -149,6 +137,15 @@ void QdsNewDialog::setProjectLocation(const QString &location)
     m_wizard.setProjectLocation(m_qmlProjectLocation);
 }
 
+void QdsNewDialog::setHasCMakeGeneration(bool haveCmakeGen)
+{
+    if (m_qmlHasCMakeGeneration == haveCmakeGen)
+        return;
+
+    m_qmlHasCMakeGeneration = haveCmakeGen;
+    emit hasCMakeGenerationChanged();
+}
+
 void QdsNewDialog::onStatusMessageChanged(Utils::InfoLabel::InfoType type, const QString &message)
 {
     switch (type) {
@@ -198,26 +195,42 @@ void QdsNewDialog::onWizardCreated(QStandardItemModel *screenSizeModel, QStandar
         m_screenSizeModel->setBackendModel(screenSizeModel);
 
     if (styleModel)
-        m_styleModel->setBackendModel(styleModel);
+        m_styleModel->setSourceModel(styleModel);
+
+    if (!m_currentPreset) {
+        qWarning() << "Wizard has been created but there is no Preset selected!";
+        return;
+    }
 
     auto userPreset = m_currentPreset->asUserPreset();
 
     if (m_qmlDetailsLoaded) {
-        if (m_currentPreset->isUserPreset()) {
-            if (m_wizard.haveVirtualKeyboard())
-                setUseVirtualKeyboard(userPreset->useQtVirtualKeyboard);
+        setHasCMakeGeneration(m_wizard.hasCMakeGeneration());
 
-            if (m_wizard.haveTargetQtVersion()) {
-                int index = m_wizard.targetQtVersionIndex(userPreset->qtVersion);
-                if (index != -1)
-                    setTargetQtVersionIndex(index);
-            }
+        if (m_currentPreset->isUserPreset()) {
+            if (getHaveVirtualKeyboard())
+                setUseVirtualKeyboard(userPreset->useQtVirtualKeyboard);
+            if (hasCMakeGeneration())
+                setEnableCMakeGeneration(userPreset->enableCMakeGeneration);
+
+            setStyleName(userPreset->styleName);
         } else {
-            if (m_wizard.haveTargetQtVersion()) {
-                int index = m_wizard.targetQtVersionIndex();
-                if (index != -1)
-                    setTargetQtVersionIndex(index);
-            }
+            if (getHaveVirtualKeyboard())
+                setUseVirtualKeyboard(m_wizard.virtualKeyboardUsed());
+            if (hasCMakeGeneration())
+                setEnableCMakeGeneration(m_wizard.cmakeGenerationEnabled());
+
+            setStyleName(m_wizard.styleName());
+        }
+
+        m_targetQtVersions.clear();
+        if (m_wizard.haveTargetQtVersion()) {
+            m_targetQtVersions = m_wizard.targetQtVersionNames();
+            int index = m_currentPreset->isUserPreset() ? m_wizard.targetQtVersionIndex(userPreset->qtVersion)
+                                                        : m_wizard.targetQtVersionIndex();
+            emit targetQtVersionsChanged();
+            if (index != -1)
+                setTargetQtVersionIndex(index);
         }
 
         emit haveVirtualKeyboardChanged();
@@ -226,21 +239,26 @@ void QdsNewDialog::onWizardCreated(QStandardItemModel *screenSizeModel, QStandar
         updateScreenSizes();
 
         setProjectName(m_qmlProjectName);
-        setProjectLocation(m_qmlProjectLocation.toString());
+        setProjectLocation(m_qmlProjectLocation.toUrlishString());
     }
+}
 
-    if (m_qmlStylesLoaded && m_wizard.haveStyleModel()) {
-        if (m_currentPreset->isUserPreset()) {
-            int index = m_wizard.styleIndex(userPreset->styleName);
-            if (index != -1)
-                setStyleIndex(index);
-        } else {
-            /* NOTE: For a builtin preset, we don't need to set style index. That's because defaults
-             *       will be loaded from the backend Wizard.
-             */
-        }
-        m_styleModel->reset();
-    }
+void QdsNewDialog::setStyleName(const QString &newStyleName)
+{
+    if (m_styleName == newStyleName)
+        return;
+
+    m_styleName = newStyleName;
+    emit styleNameChanged();
+}
+
+void QdsNewDialog::setEnableCMakeGeneration(bool newQmlEnableCMakeGeneration)
+{
+    if (m_qmlEnableCMakeGeneration == newQmlEnableCMakeGeneration)
+        return;
+
+    m_qmlEnableCMakeGeneration = newQmlEnableCMakeGeneration;
+    emit enableCMakeGenerationChanged();
 }
 
 QString QdsNewDialog::currentPresetQmlPath() const
@@ -277,44 +295,9 @@ int QdsNewDialog::getTargetQtVersionIndex() const
     return m_qmlTargetQtVersionIndex;
 }
 
-void QdsNewDialog::setStyleIndex(int index)
-{
-    if (!m_qmlStylesLoaded)
-        return;
-
-    if (index == -1) {
-        m_qmlStyleIndex = index;
-        return;
-    }
-
-    m_qmlStyleIndex = index;
-    int actualIndex = m_styleModel->actualIndex(m_qmlStyleIndex);
-    QTC_ASSERT(actualIndex >= 0, return);
-
-    m_wizard.setStyleIndex(actualIndex);
-}
-
 int QdsNewDialog::getStyleIndex() const
 {
-    /**
-     * m_wizard.styleIndex property is the wizard's (backend's) value of the style index.
-     * The initial value (saved in the wizard.json) is read from there. Any subsequent reads of
-     * the style index should use m_styleIndex, which is the QML's style index property. Setting
-     * the style index should update both the m_styleIndex and the backend. In this regard, the
-     * QdsNewDialog's m_styleIndex acts as some kind of cache.
-    */
-
-    if (!m_qmlStylesLoaded)
-        return -1;
-
-    if (m_qmlStyleIndex == -1) {
-        int actualIndex = m_wizard.styleIndex();
-        // Not nice, get sets the property... m_qmlStyleIndex acts like a cache.
-        m_qmlStyleIndex = m_styleModel->filteredIndex(actualIndex);
-        return m_qmlStyleIndex;
-    }
-
-    return m_styleModel->actualIndex(m_qmlStyleIndex);
+    return m_styleModel->findSourceIndex(m_styleName);
 }
 
 void QdsNewDialog::setUseVirtualKeyboard(bool value)
@@ -331,7 +314,7 @@ void QdsNewDialog::setWizardFactories(QList<Core::IWizardFactory *> factories_,
 {
     Utils::Id platform = Utils::Id::fromSetting("Desktop");
 
-    WizardFactories factories{factories_, m_dialog, platform};
+    WizardFactories factories{factories_, platform};
 
     std::vector<UserPresetData> recents = m_recentsStore.fetchAll();
     std::vector<UserPresetData> userPresets =  m_userPresetsStore.fetchAll();
@@ -349,10 +332,10 @@ void QdsNewDialog::setWizardFactories(QList<Core::IWizardFactory *> factories_,
     const Core::IWizardFactory *first = factories.front();
     Utils::FilePath projectLocation = first->runPath(defaultLocation);
 
-    m_qmlProjectName = uniqueProjectName(projectLocation.toString());
+    m_qmlProjectName = uniqueProjectName(projectLocation.toUrlishString());
     emit projectNameChanged(); // So that QML knows to update the field
 
-    m_qmlProjectLocation = Utils::FilePath::fromString(QDir::toNativeSeparators(projectLocation.toString()));
+    m_qmlProjectLocation = Utils::FilePath::fromString(QDir::toNativeSeparators(projectLocation.toUrlishString()));
     emit projectLocationChanged(); // So that QML knows to update the field
 
     /* NOTE:
@@ -383,7 +366,7 @@ QString QdsNewDialog::recentsTabName() const
 
 QString QdsNewDialog::qmlPath() const
 {
-    return Core::ICore::resourcePath("qmldesigner/newprojectdialog/NewProjectDialog.qml").toString();
+    return Core::ICore::resourcePath("qmldesigner/newprojectdialog/NewProjectDialog.qml").toUrlishString();
 }
 
 void QdsNewDialog::showDialog()
@@ -401,47 +384,44 @@ bool QdsNewDialog::getHaveTargetQtVersion() const
     return m_wizard.haveTargetQtVersion();
 }
 
+bool QdsNewDialog::hasCMakeGeneration() const
+{
+    return m_qmlHasCMakeGeneration;
+}
+
 void QdsNewDialog::accept()
 {
     CreateProject create{m_wizard};
+    // Get a snapshot of the preset before hiding the dialog
+    UserPresetData preset = currentUserPresetData(m_currentPreset->displayName());
 
     m_dialog->hide();
     create.withName(m_qmlProjectName)
         .atLocation(m_qmlProjectLocation)
         .withScreenSizes(m_qmlScreenSizeIndex, m_qmlCustomWidth, m_qmlCustomHeight)
-        .withStyle(m_qmlStyleIndex)
+        .withStyle(getStyleIndex())
         .useQtVirtualKeyboard(m_qmlUseVirtualKeyboard)
+        .enableCMakeGeneration(m_qmlEnableCMakeGeneration)
         .saveAsDefaultLocation(m_qmlSaveAsDefaultLocation)
         .withTargetQtVersion(m_qmlTargetQtVersionIndex)
         .execute();
 
-    std::shared_ptr<PresetItem> item = m_wizard.preset();
-    QString customSizeName = m_qmlCustomWidth + " x " + m_qmlCustomHeight;
-
-    UserPresetData preset = currentUserPresetData(m_currentPreset->displayName());
     m_recentsStore.save(preset);
 
     m_dialog->close();
-    m_dialog->deleteLater();
-    m_dialog = nullptr;
 }
 
 void QdsNewDialog::reject()
 {
-    m_screenSizeModel->setBackendModel(nullptr);
-    m_styleModel->setBackendModel(nullptr);
-    m_wizard.destroyWizard();
-
     m_dialog->close();
-    m_dialog = nullptr;
 }
 
 QString QdsNewDialog::chooseProjectLocation()
 {
-    Utils::FilePath newPath = Utils::FileUtils::getExistingDirectory(m_dialog, tr("Choose Directory"),
+    Utils::FilePath newPath = Utils::FileUtils::getExistingDirectory(tr("Choose Directory"),
                                                                      m_qmlProjectLocation);
 
-    return QDir::toNativeSeparators(newPath.toString());
+    return QDir::toNativeSeparators(newPath.toUrlishString());
 }
 
 void QdsNewDialog::setSelectedPreset(int selection)
@@ -465,23 +445,29 @@ UserPresetData QdsNewDialog::currentUserPresetData(const QString &displayName) c
     QString targetQtVersion = "";
     QString styleName = "";
     bool useVirtualKeyboard = false;
+    bool enableCMakeGeneration = false;
 
     if (m_wizard.haveTargetQtVersion())
         targetQtVersion = m_wizard.targetQtVersionName(m_qmlTargetQtVersionIndex);
 
     if (m_wizard.haveStyleModel())
-        styleName = m_wizard.styleName(m_qmlStyleIndex);
+        styleName = m_wizard.styleNameAt(getStyleIndex());
 
     if (m_wizard.haveVirtualKeyboard())
         useVirtualKeyboard = m_qmlUseVirtualKeyboard;
 
-    UserPresetData preset = {m_currentPreset->categoryId,
-                             m_currentPreset->wizardName,
-                             displayName,
-                             screenSize,
-                             useVirtualKeyboard,
-                             targetQtVersion,
-                             styleName};
+    if (m_wizard.hasCMakeGeneration())
+        enableCMakeGeneration = m_qmlEnableCMakeGeneration;
+
+    UserPresetData preset{
+        m_currentPreset->categoryId,
+        m_currentPreset->wizardName,
+        displayName,
+        screenSize,
+        useVirtualKeyboard,
+        enableCMakeGeneration,
+        targetQtVersion,
+        styleName};
 
     return preset;
 }
@@ -491,7 +477,7 @@ void QdsNewDialog::savePresetDialogAccept()
     UserPresetData preset = currentUserPresetData(m_qmlPresetName);
 
     if (!m_userPresetsStore.save(preset)) {
-        QMessageBox::warning(m_dialog,
+        QMessageBox::warning(m_dialog.get(),
                              tr("Save Preset"),
                              tr("A preset with this name already exists."));
         return;
