@@ -1,32 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include <QtTest>
 #include <QDebug>
-#include <QTextDocument>
+#include <QTest>
 #include <QTextCursor>
+#include <QTextDocument>
 
 #include <cplusplus/AST.h>
 #include <cplusplus/ASTVisitor.h>
@@ -57,19 +35,19 @@ class tst_Semantic: public QObject
 {
     Q_OBJECT
 
-    QSharedPointer<Control> control;
+    std::shared_ptr<Control> control;
 
 public:
     tst_Semantic()
         : control(new Control)
-    { control->setDiagnosticClient(&diag); }
+    { control->setDiagnosticClient(&diag, true); }
 
     TranslationUnit *parse(const QByteArray &source,
                            TranslationUnit::ParseMode mode,
                            LanguageFeatures features)
     {
         const StringLiteral *fileId = control->stringLiteral("<stdin>");
-        TranslationUnit *unit = new TranslationUnit(control.data(), fileId);
+        TranslationUnit *unit = new TranslationUnit(control.get(), fileId);
         unit->setSource(source.constData(), source.length());
         unit->setLanguageFeatures(features);
         unit->parse(mode);
@@ -112,21 +90,21 @@ public:
             : errorCount(0)
         { }
 
-        virtual void report(int /*level*/,
-                            const StringLiteral *fileName,
-                            unsigned line, unsigned column,
-                            const char *format, va_list ap)
+        void report(int /*level*/,
+                    const StringLiteral *fileName,
+                    int line, int column,
+                    const char *format, va_list ap) override
         {
             ++errorCount;
 
 #ifndef NO_PARSER_OR_SEMANTIC_ERROR_MESSAGES
-            qDebug() << fileName->chars()<<':'<<line<<':'<<column<<' '<<QString().vsprintf(format, ap);
+            qDebug() << fileName->chars()<<':'<<line<<':'<<column<<' ' << QString::vasprintf(format, ap);
 #else
-            Q_UNUSED(fileName);
-            Q_UNUSED(line);
-            Q_UNUSED(column);
-            Q_UNUSED(format);
-            Q_UNUSED(ap);
+            Q_UNUSED(fileName)
+            Q_UNUSED(line)
+            Q_UNUSED(column)
+            Q_UNUSED(format)
+            Q_UNUSED(ap)
 #endif
         }
     };
@@ -144,7 +122,7 @@ public:
         features.cxx11Enabled = enableCxx11;
         features.cxxEnabled = true;
         diag.errorCount = 0; // reset the error count.
-        TranslationUnit *unit = parse(source, TranslationUnit::ParseTranlationUnit, features);
+        TranslationUnit *unit = parse(source, TranslationUnit::ParseTranslationUnit, features);
         QSharedPointer<Document> doc(new Document(unit));
         doc->check();
         doc->errorCount = diag.errorCount;
@@ -198,7 +176,7 @@ void tst_Semantic::function_declaration_1()
 {
     QSharedPointer<Document> doc = document("void foo();");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -206,11 +184,11 @@ void tst_Semantic::function_declaration_1()
     FullySpecifiedType declTy = decl->type();
     Function *funTy = declTy->asFunctionType();
     QVERIFY(funTy);
-    QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 0U);
+    QVERIFY(funTy->returnType()->asVoidType());
+    QCOMPARE(funTy->argumentCount(), 0);
     QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
-    QVERIFY(decl->name()->isNameId());
+    QVERIFY(decl->name()->asNameId());
     const Identifier *funId = decl->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -222,7 +200,7 @@ void tst_Semantic::function_declaration_2()
 {
     QSharedPointer<Document> doc = document("void foo(const QString &s);");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -230,8 +208,8 @@ void tst_Semantic::function_declaration_2()
     FullySpecifiedType declTy = decl->type();
     Function *funTy = declTy->asFunctionType();
     QVERIFY(funTy);
-    QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QVERIFY(funTy->returnType()->asVoidType());
+    QCOMPARE(funTy->argumentCount(), 1);
     QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
     // check the formal argument.
@@ -251,7 +229,7 @@ void tst_Semantic::function_declaration_2()
 
     // check the type of the formal argument
     FullySpecifiedType argTy = arg->type();
-    QVERIFY(argTy->isReferenceType());
+    QVERIFY(argTy->asReferenceType());
     QVERIFY(argTy->asReferenceType()->elementType().isConst());
     NamedType *namedTy = argTy->asReferenceType()->elementType()->asNamedType();
     QVERIFY(namedTy);
@@ -261,7 +239,7 @@ void tst_Semantic::function_declaration_2()
     QCOMPARE(QByteArray(namedTypeId->chars(), namedTypeId->size()),
              QByteArray("QString"));
 
-    QVERIFY(decl->name()->isNameId());
+    QVERIFY(decl->name()->asNameId());
     const Identifier *funId = decl->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -351,7 +329,7 @@ void tst_Semantic::function_declaration_ref_qualifier()
         return;
     }
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -359,8 +337,8 @@ void tst_Semantic::function_declaration_ref_qualifier()
     FullySpecifiedType declTy = decl->type();
     Function *funTy = declTy->asFunctionType();
     QVERIFY(funTy);
-    QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 0U);
+    QVERIFY(funTy->returnType()->asVoidType());
+    QCOMPARE(funTy->argumentCount(), 0);
 
     // check the ref-qualifier
     QCOMPARE(funTy->refQualifier(), refQualifier);
@@ -370,15 +348,15 @@ void tst_Semantic::function_definition_1()
 {
     QSharedPointer<Document> doc = document("void foo() {}");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Function *funTy = doc->globals->memberAt(0)->asFunction();
     QVERIFY(funTy);
-    QVERIFY(funTy->returnType()->isVoidType());
-    QCOMPARE(funTy->argumentCount(), 0U);
+    QVERIFY(funTy->returnType()->asVoidType());
+    QCOMPARE(funTy->argumentCount(), 0);
     QCOMPARE(funTy->refQualifier(), Function::NoRefQualifier);
 
-    QVERIFY(funTy->name()->isNameId());
+    QVERIFY(funTy->name()->asNameId());
     const Identifier *funId = funTy->name()->asNameId()->identifier();
     QVERIFY(funId);
 
@@ -398,7 +376,7 @@ void tst_Semantic::nested_class_1()
 "};\n"
     );
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     Class *classObject = doc->globals->memberAt(0)->asClass();
     QVERIFY(classObject);
@@ -407,8 +385,8 @@ void tst_Semantic::nested_class_1()
     QVERIFY(classObjectNameId);
     const Identifier *objectId = classObjectNameId->identifier();
     QCOMPARE(QByteArray(objectId->chars(), objectId->size()), QByteArray("Object"));
-    QCOMPARE(classObject->baseClassCount(), 0U);
-    QCOMPARE(classObject->memberCount(), 2U);
+    QCOMPARE(classObject->baseClassCount(), 0);
+    QCOMPARE(classObject->memberCount(), 2);
 
     Class *classObjectData = doc->globals->memberAt(1)->asClass();
     QVERIFY(classObjectData);
@@ -439,7 +417,7 @@ void tst_Semantic::alias_declaration_1()
                 , false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl->name());
@@ -448,7 +426,7 @@ void tst_Semantic::alias_declaration_1()
 
     QVERIFY(decl->isTypedef());
     QVERIFY(decl->type().isTypedef());
-    QVERIFY(decl->type()->isIntegerType());
+    QVERIFY(decl->type()->asIntegerType());
 }
 
 void tst_Semantic::typedef_1()
@@ -464,11 +442,11 @@ void tst_Semantic::typedef_1()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 3U);
+    QCOMPARE(doc->globals->memberCount(), 3);
 
     Class *anonStruct = doc->globals->memberAt(0)->asClass();
     QVERIFY(anonStruct);
-    QCOMPARE(anonStruct->memberCount(), 2U);
+    QCOMPARE(anonStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
@@ -493,16 +471,16 @@ void tst_Semantic::typedef_2()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 3U);
+    QCOMPARE(doc->globals->memberCount(), 3);
 
     Class *_pointStruct= doc->globals->memberAt(0)->asClass();
     QVERIFY(_pointStruct);
-    QCOMPARE(_pointStruct->memberCount(), 2U);
+    QCOMPARE(_pointStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
     QVERIFY(typedefPointDecl->isTypedef());
-    QVERIFY(typedefPointDecl->type()->isNamedType());
+    QVERIFY(typedefPointDecl->type()->asNamedType());
     QCOMPARE(typedefPointDecl->type()->asNamedType()->name(), _pointStruct->name());
 
     Function *mainFun = doc->globals->memberAt(2)->asFunction();
@@ -518,16 +496,16 @@ void tst_Semantic::typedef_3()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     Class *_pointStruct= doc->globals->memberAt(0)->asClass();
     QVERIFY(_pointStruct);
-    QCOMPARE(_pointStruct->memberCount(), 2U);
+    QCOMPARE(_pointStruct->memberCount(), 2);
 
     Declaration *typedefPointDecl = doc->globals->memberAt(1)->asDeclaration();
     QVERIFY(typedefPointDecl);
     QVERIFY(typedefPointDecl->isTypedef());
-    QVERIFY(typedefPointDecl->type()->isPointerType());
+    QVERIFY(typedefPointDecl->type()->asPointerType());
     QCOMPARE(typedefPointDecl->type()->asPointerType()->elementType()->asClassType(),
              _pointStruct);
 }
@@ -539,20 +517,20 @@ void tst_Semantic::const_1()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
-    QVERIFY(decl->type()->isFunctionType());
+    QVERIFY(decl->type()->asFunctionType());
     Function *funTy = decl->type()->asFunctionType();
-    QVERIFY(funTy->returnType()->isIntegerType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QVERIFY(funTy->returnType()->asIntegerType());
+    QCOMPARE(funTy->argumentCount(), 1);
     Argument *arg = funTy->argumentAt(0)->asArgument();
     QVERIFY(arg);
     QVERIFY(! arg->type().isConst());
-    QVERIFY(arg->type()->isPointerType());
+    QVERIFY(arg->type()->asPointerType());
     QVERIFY(arg->type()->asPointerType()->elementType().isConst());
-    QVERIFY(arg->type()->asPointerType()->elementType()->isIntegerType());
+    QVERIFY(arg->type()->asPointerType()->elementType()->asIntegerType());
 }
 
 void tst_Semantic::const_2()
@@ -562,27 +540,27 @@ void tst_Semantic::const_2()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
-    QVERIFY(decl->type()->isFunctionType());
+    QVERIFY(decl->type()->asFunctionType());
     Function *funTy = decl->type()->asFunctionType();
-    QVERIFY(funTy->returnType()->isIntegerType());
-    QCOMPARE(funTy->argumentCount(), 1U);
+    QVERIFY(funTy->returnType()->asIntegerType());
+    QCOMPARE(funTy->argumentCount(), 1);
     Argument *arg = funTy->argumentAt(0)->asArgument();
     QVERIFY(arg);
     QVERIFY(arg->type().isConst());
-    QVERIFY(arg->type()->isPointerType());
+    QVERIFY(arg->type()->asPointerType());
     QVERIFY(! arg->type()->asPointerType()->elementType().isConst());
-    QVERIFY(arg->type()->asPointerType()->elementType()->isIntegerType());
+    QVERIFY(arg->type()->asPointerType()->elementType()->asIntegerType());
 }
 
 void tst_Semantic::pointer_to_function_1()
 {
     QSharedPointer<Document> doc = document("void (*QtSomething)();");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Declaration *decl = doc->globals->memberAt(0)->asDeclaration();
     QVERIFY(decl);
@@ -604,7 +582,7 @@ void tst_Semantic::template_instance_1()
 {
     QSharedPointer<Document> doc = document("template <typename _Tp> class QList { void append(const _Tp &value); };");
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     Template *templ = doc->globals->memberAt(0)->asTemplate();
     QVERIFY(templ);
@@ -612,7 +590,7 @@ void tst_Semantic::template_instance_1()
     Declaration *decl = templ->memberAt(1)->asClass()->memberAt(0)->asDeclaration();
     QVERIFY(decl);
 
-    FullySpecifiedType templArgs[] = {control->integerType(IntegerType::Int)};
+    TemplateArgument templArgs[] = {FullySpecifiedType(control->integerType(IntegerType::Int))};
     const Name *templId = control->templateNameId(control->identifier("QList"), false, templArgs, 1);
 
     FullySpecifiedType genTy = DeprecatedGenTemplateInstance::instantiate(templId, decl, control);
@@ -716,17 +694,17 @@ void tst_Semantic::objcClass_1()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 2U);
+    QCOMPARE(doc->globals->memberCount(), 2);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
     QVERIFY(iface->isInterface());
-    QCOMPARE(iface->memberCount(), 2U);
+    QCOMPARE(iface->memberCount(), 2);
 
     ObjCClass *impl = doc->globals->memberAt(1)->asObjCClass();
     QVERIFY(impl);
     QVERIFY(!impl->isInterface());
-    QCOMPARE(impl->memberCount(), 3U);
+    QCOMPARE(impl->memberCount(), 3);
 
     ObjCMethod *allocMethod = impl->memberAt(0)->asObjCMethod();
     QVERIFY(allocMethod);
@@ -750,19 +728,19 @@ void tst_Semantic::objcSelector_1()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
     QVERIFY(iface->isInterface());
-    QCOMPARE(iface->memberCount(), 1U);
+    QCOMPARE(iface->memberCount(), 1);
 
     Declaration *decl = iface->memberAt(0)->asDeclaration();
     QVERIFY(decl);
     QVERIFY(decl->name());
     const SelectorNameId *selId = decl->name()->asSelectorNameId();
     QVERIFY(selId);
-    QCOMPARE(selId->nameCount(), 3U);
+    QCOMPARE(selId->nameCount(), 3);
     QCOMPARE(selId->nameAt(0)->identifier()->chars(), "a");
     QCOMPARE(selId->nameAt(1)->identifier()->chars(), "b");
     QCOMPARE(selId->nameAt(2)->identifier()->chars(), "c");
@@ -779,7 +757,7 @@ public:
         return selectors;
     }
 
-    virtual bool visit(ObjCSelectorAST *ast) {selectors.append(ast); return false;}
+    bool visit(ObjCSelectorAST *ast) override {selectors.append(ast); return false;}
 
 private:
     QList<ObjCSelectorAST *> selectors;
@@ -796,11 +774,11 @@ void tst_Semantic::objcSelector_2()
                                             true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 
     ObjCClass *iface = doc->globals->memberAt(0)->asObjCClass();
     QVERIFY(iface);
-    QCOMPARE(iface->memberCount(), 1U);
+    QCOMPARE(iface->memberCount(), 1);
 
     QList<ObjCSelectorAST*>selectors = CollectSelectors(doc->unit)();
     QCOMPARE(selectors.size(), 2);
@@ -810,7 +788,7 @@ void tst_Semantic::objcSelector_2()
 
     const SelectorNameId *selId = sel->name->asSelectorNameId();
     QVERIFY(selId);
-    QCOMPARE(selId->nameCount(), 3U);
+    QCOMPARE(selId->nameCount(), 3);
     QCOMPARE(selId->nameAt(0)->identifier()->chars(), "a");
     QCOMPARE(selId->nameAt(1)->identifier()->chars(), "b");
     QCOMPARE(selId->nameAt(2)->identifier()->chars(), "c");
@@ -827,7 +805,7 @@ void tst_Semantic::q_enum_1()
                                             false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     QVERIFY(doc->unit);
     TranslationUnitAST *xUnit = doc->unit->ast()->asTranslationUnit();
     QVERIFY(xUnit);
@@ -851,7 +829,7 @@ void tst_Semantic::lambda_1()
                                             "}\n", false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 }
 
 void tst_Semantic::lambda_2()
@@ -864,15 +842,15 @@ void tst_Semantic::lambda_2()
                 , false, false, true);
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Class *A = doc->globals->memberAt(0)->asClass();
     QVERIFY(A);
-    QCOMPARE(A->memberCount(), 1U);
+    QCOMPARE(A->memberCount(), 1);
     Declaration *d = A->memberAt(0)->asDeclaration();
     QCOMPARE(d->name()->identifier()->chars(), "f");
     Function *ty = d->type()->asFunctionType();
     QVERIFY(ty);
-    QCOMPARE(ty->argumentCount(), 1U);
+    QCOMPARE(ty->argumentCount(), 1);
     Argument *arg = ty->argumentAt(0)->asArgument();
     QVERIFY(arg);
     const StringLiteral *init = arg->initializer();
@@ -888,7 +866,7 @@ void tst_Semantic::diagnostic_error()
                                             false, false);
 
     QCOMPARE(doc->errorCount, 1U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
 }
 
 namespace {
@@ -916,10 +894,10 @@ void tst_Semantic::enum_constantValue1()
                                             );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 3U);
+    QCOMPARE(e->memberCount(), 3);
 
     testEnumaratorDeclarator(e, 0, "0");
     testEnumaratorDeclarator(e, 1, "1");
@@ -937,10 +915,10 @@ void tst_Semantic::enum_constantValue2()
                                             );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 3U);
+    QCOMPARE(e->memberCount(), 3);
 
     testEnumaratorDeclarator(e, 0, "10");
     testEnumaratorDeclarator(e, 1, "11");
@@ -958,10 +936,10 @@ void tst_Semantic::enum_constantValue3()
                                             );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 3U);
+    QCOMPARE(e->memberCount(), 3);
 
     testEnumaratorDeclarator(e, 0, "0");
     testEnumaratorDeclarator(e, 1, "10");
@@ -981,10 +959,10 @@ void tst_Semantic::enum_constantValue4()
                                             );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 5U);
+    QCOMPARE(e->memberCount(), 5);
 
     testEnumaratorDeclarator(e, 0, "0");
     testEnumaratorDeclarator(e, 1, "E1+10");
@@ -1006,10 +984,10 @@ void tst_Semantic::enum_constantValue5()
                                             );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 5U);
+    QCOMPARE(e->memberCount(), 5);
 
     testEnumaratorDeclarator(e, 0, "0");
     testEnumaratorDeclarator(e, 1, "0");
@@ -1030,10 +1008,10 @@ void tst_Semantic::enum_constantValueNegative()
     );
 
     QCOMPARE(doc->errorCount, 0U);
-    QCOMPARE(doc->globals->memberCount(), 1U);
+    QCOMPARE(doc->globals->memberCount(), 1);
     Enum *e = doc->globals->memberAt(0)->asEnum();
     QVERIFY(e);
-    QCOMPARE(e->memberCount(), 4U);
+    QCOMPARE(e->memberCount(), 4);
 
     testEnumaratorDeclarator(e, 0, "-2");
     testEnumaratorDeclarator(e, 1, "-1");
@@ -1041,5 +1019,5 @@ void tst_Semantic::enum_constantValueNegative()
     testEnumaratorDeclarator(e, 3, "1");
 }
 
-QTEST_MAIN(tst_Semantic)
+QTEST_GUILESS_MAIN(tst_Semantic)
 #include "tst_semantic.moc"
